@@ -1,19 +1,28 @@
 # coding: utf-8
 
 ## const default value
+import queue
+
+import time
 import base64
 
 #
 try:
-    from aiges_embed import ResponseData, Response, DataListNode, DataListCls
+    from aiges_embed import ResponseData, Response, DataListNode, DataListCls, SessionCreateResponse
 except:
-    from aiges.dto import Response, ResponseData, DataListNode, DataListCls, DataAudio
+    from aiges.dto import Response, ResponseData, DataListNode, DataListCls, DataAudio, SessionCreateResponse
 
 from jinja2 import Template
 import json
 import os
 from aiges.utils.log import log
+import threading
 from pprint import pprint
+from aiges.types import *
+from aiges.stream import *
+import queue
+from threading import Lock
+from multiprocessing import Process
 
 SUB = "ase"
 CALL = "atmos"
@@ -23,11 +32,6 @@ HOSTS = [
 ]
 ROUTEKEY = []
 
-STRING = 0
-AUDIO = 1
-IMAGE = 2
-VIDEO = 3
-
 type_map = {
     "text": STRING,
     "audio": AUDIO,
@@ -179,9 +183,12 @@ class JsonBodyField(PayloadField):
         else:
             return base64.b64encode(self.value)
 
+
 class StringBodyField(PayloadField):
-    def __init__(self, key, value="", need_base64=False):
+    def __init__(self, key, value=b"", need_base64=False):
         super(StringBodyField, self).__init__(key, STRING)
+        if not isinstance(value, bytes):
+            raise Exception("StringBodyField value must be bytes String...")
         self.value = value
         self.data_type = STRING
         self.need_base64 = need_base64
@@ -396,6 +403,7 @@ class WrapperBase(metaclass=Metaclass):
         self.inputs_test_values = {}
         self.params_test_values = {}
         self.respData = []
+        self.session = SessionManager()
 
     def schema(self):
         s = Template(tpl)
@@ -676,6 +684,10 @@ class WrapperBase(metaclass=Metaclass):
         raise NotImplementedError(
             "Please Inplement Wrapper Class Method: wrapperOnceExec(cls, usrTag: str, params: {}, reqData: [], respData: [], psrIds: [], psrCnt: int) ")
 
+    def wrapperOnceExecAsync(cls, params: {}, reqData: DataListCls, sid: str) -> Response:
+        raise NotImplementedError(
+            "Please Inplement Wrapper Class Method: wrapperOnceExec(cls, usrTag: str, params: {}, reqData: [], respData: [], psrIds: [], psrCnt: int) ")
+
     def wrapperError(cls, ret: int) -> str:
         if ret == 100:
             return "This is a  error return"
@@ -685,8 +697,11 @@ class WrapperBase(metaclass=Metaclass):
         保留接口
     '''
 
-    def wrapperCreate(cls, usrTag: str, params: [], psrIds: [], psrCnt: int) -> str:
-        return ""
+    def wrapperCreate(cls, params: {}, sid: str) -> SessionCreateResponse:
+        print(params)
+
+        print(sid)
+        return
 
     '''
         保留接口
@@ -716,17 +731,30 @@ class WrapperBase(metaclass=Metaclass):
             log.error("Please return content in response.list")
             return False
 
+        for d in resp.list:
+            # check Response data 类型
+            if not isinstance(d.data, memoryview) and not isinstance(d.data, bytes):
+                log.error("ResponseData 's data field must be  bytes or memoryview")
+                return False
+
         # check 响应key是否重复
         keys = [r.key for r in resp.list]
         set_keys = set(keys)
+        log.info("response keys: %s", str(keys))
         if not len(keys) == len(set_keys):
             log.error("response list keys must be unique...")
             log.error("invalid keys %s" % str(keys))
             return False
 
-        print(keys)
+        return True
 
-    def run(self):
+    def run(self, stream=False):
+        if not stream:
+            self.run_once()
+        else:
+            self.run_stream()
+
+    def run_once(self):
         # 1. 模拟调用初始化引擎
         #  传入配置当前模拟为空
         self.wrapperInit(self.config)
@@ -753,10 +781,15 @@ class WrapperBase(metaclass=Metaclass):
             req.list = tmp
             # 3. 模拟调用 exec，并返回数据
             response = self.wrapperOnceExec(params, req)
-            self.check_resp(response)
+            if self.check_resp(response):
+                log.info("wrapper.py has been verified... Congratulations ...!")
+            else:
+                log.error("Sorry, Please Check The Log Output Above ...")
         except Exception as e:
             # 4. 模拟检查 wrapperOnceExec返回
             log.error(e)
             self.wrapperError(-1)
 
-        # todo respData 检查
+    def run_stream(self):
+        raise NotImplementedError("Not implement, will be ok next verion (maybe in v0.5.0+)")
+        pass
